function chart_pos_order (widget) {
    "use strict";

    var model = openerp;

    widget.ChartPosOrderWidget = widget.Base.extend({
        template: 'ChartPosOrder',
        data: [],
        events: {
            'click .month': 'showMonth',
            'click .week': 'showWeek',
            'click .days': 'showDays',
        },

        init: function (parent) {
            this._super(parent, {
                width: 6,
                height: 4
            });
        },

        start: function () {
            var self = this;
            self.fetchInitial();
        },

        fetchInitial:function() {
            var self = this;
            self.$el.block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#FAFAFA'
                }
            });

            self.$el.find('.widget-content.widget-loading').css('display','flex');

            self.fetchCurrentUser().then(function (CurrentUser) {
                return CurrentUser;
            }).then(function (CurrentUser) {
                self.CurrentUser = CurrentUser;
                return self.fetchResUser(CurrentUser);
            }).then(function (ResUser) {
                self.ResUser = ResUser;
                return self.fetchAccountJournal();
            }).then(function (AccountJournal) {
                self.AccountJournal = AccountJournal;
                return self.fetchPosOrder();
            }).then(function (PosOrder) {
                self.PosOrder = PosOrder;
                return self.fetchAccountInvoice();
            }).then(function (AccountInvoice) {
                self.AccountInvoice = AccountInvoice;
                return self.showMonth();
            });
        },

        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        fetchResUser: function(id) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','store_id'];
            var domain = [['id','=',id]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountJournal: function() {
            var self = this;
            var defer = $.Deferred();
            var store_ids = self.ResUser[0].store_id[0];
            var fields = ['id', 'name', 'store_ids'];
            var domain = [['type','=','sale'],['store_ids','in',store_ids]];
            var AccountJournal = new model.web.Model('account.journal');
            AccountJournal.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },
        
        fetchPosOrder: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(this.AccountJournal, function (item) {
                return item.id;
            }));
            var fields = ['id','date_order','amount_total'];
            var domain = [['state','not in',['draft','cancel']],['sale_journal','in',journal_ids] ];
            var PosOrder = new model.web.Model('pos.order');
            PosOrder.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountInvoice: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(this.AccountJournal, function (item) {
                return item.id;
            }));
            var fields = ['id', 'name', 'date_invoice', 'amount_total'];
            var domain = [['state', 'not in', ['draft','cancel']],['type','=','out_invoice'],['journal_id','in',journal_ids]];
            var AccountInvoice = new model.web.Model('account.invoice');
            AccountInvoice.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },

        getMonthPosOrder:function(mes) {
            var self = this;
            if (mes < 10){
                var fecha = moment().format('YYYY')+'-'+'0'+mes;    
            }else{
                var fecha = moment().format('YYYY')+'-'+mes;    
            }
            return _.flatten(_.filter(self.PosOrder,function (inv) {
                return moment(inv.date_order).format('YYYY-MM') === fecha;
            }));
        },

        getWeekPosOrder:function(day) {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.PosOrder,function (inv) {
                return moment(inv.date_order).week() === week & moment(inv.date_order).isoWeekday() === day;
            }));
        },

        getDaysPosOrder:function(day) {
            var self = this;
            var date = moment().subtract(day, 'days').format('YYYY-MM-DD');
            return _.flatten(_.filter(self.PosOrder,function (inv) {
                return moment(inv.date_order).format('YYYY-MM-DD') === date;
            }));
        },

        getMonthAccountInvoice:function(mes) {
            var self = this;
            if (mes < 10){
                var fecha = moment().format('YYYY')+'-'+'0'+mes;    
            }else{
                var fecha = moment().format('YYYY')+'-'+mes;    
            }
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM') === fecha;
            }));
        },

        getWeekAccountInvoice:function(day) {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).week() === week & moment(inv.date_invoice).isoWeekday() === day;
            }));
        },

        getDaysAccountInvoice:function(day) {
            var self = this;
            var date = moment().subtract(day, 'days').format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountInvoice,function (inv) {
                return moment(inv.date_invoice).format('YYYY-MM-DD') === date;
            }));
        },
  
        showMonth: function() {
            var self = this;
            var order;
            var invoice;
            var title = ['Ene', 'Feb', 'Mar','Abr','May','Jun','Jul','Ago','Sep','Oct','Nov','Dic'];
            var data = [];
            for (var i = 1; i <= 12; i++) {
                order = self.getMonthPosOrder(i);
                invoice = self.getMonthAccountInvoice(i);
                var order_total = _.reduce(_.map(order,function(item) {
                    return item.amount_total;
                }),function(memo, num) {
                    return memo + num;
                },0);
                var invoice_total = _.reduce(_.map(invoice,function(item) {
                    return item.amount_total;
                }),function(memo, num) {
                    return memo + num;
                },0);
                var total = order_total + invoice_total; 
                data.push(total);
            }
            self.$el.unblock();
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.fetchChart(data,title);
        },

        showWeek: function (){
            var self = this;
            var order;
            var invoice;
            var title = ['Lunes','Martes','Miercoles','Jueves','Viernes','Sabado','Domingo'];
            var data = [];
            for (var i = 1; i <= 7; i++) {
                order = self.getWeekPosOrder(i);
                order = self.getWeekAccountInvoice(i);
                var order_total = _.reduce(_.map(order,function(item) {
                    return item.amount_total;
                }),function(memo, num) {
                    return memo + num;
                },0);
                var invoice_total = _.reduce(_.map(invoice,function(item) {
                    return item.amount_total;
                }),function(memo, num) {
                    return memo + num;
                },0);
                var total = order_total + invoice_total;
                data.push(total);
            }
            self.$el.unblock();
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.fetchChart(data,title);
        },

        showDays: function() {
            var self = this;
            var order;
            var invoice;
            var title = [];
            var data = [];
            for (var i = 15; i >= 0; i--) {
                order = self.getDaysPosOrder(i);
                invoice = self.getDaysAccountInvoice(i);
                var order_total = _.reduce(_.map(order,function(item) {
                    return item.amount_total;
                }),function(memo, num) {
                    return memo + num;
                },0);
                var invoice_total = _.reduce(_.map(invoice,function(item) {
                    return item.amount_total;
                }),function(memo, num) {
                    return memo + num;
                },0);
                var total = order_total + invoice_total;
                data.push(total);
                title.push(moment().subtract(i, 'days').format('DD / MM'));
            }

            self.$el.unblock();
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.fetchChart(data, title);
        },

        fetchChart: function (data,title) {
            var self = this;
            var label = title;
            var body = data;

            // Global method for setting Y axis number format.
            Chart.scaleService.updateScaleDefaults('linear', {
              ticks: {
                callback: function(tick) {
                    return tick.toLocaleString('de-DE');
                }
              }
            });
            Chart.defaults.global.tooltips.callbacks.label = function(tooltipItem, data) {
                var dataset = data.datasets[tooltipItem.datasetIndex];
                var datasetLabel = dataset.label || '';
                return datasetLabel +  dataset.data[tooltipItem.index].toLocaleString('de-DE');
            };
            var chart = new Chart(this.$el.find(".widget-content").find('canvas'), {
                type: 'line',
                data: {
                    labels: label,
                    datasets: [
                        {
                            label: false,
                            data: body,
                            backgroundColor: '#e3f2fd',
                            borderColor: '#64b5f6',
                            borderWidth: 2,
                            fill: true,
                        }
                    ]
                },
                options: {
                    responsive: true,
                    title: {
                        display: false,
                    },
                    hover: {
                        mode: 'nearest',
                        intersect: true
                    },
                    legend: {
                       display: false,
                    },
                    layout: {
                        padding: {
                            top: 45,
                            bottom: 40,
                            left : 10,
                            rigth: 10,
                        }
                    },
                }
            });
        },
    });
}
