function widget_account_receivable(widget) {
    "use strict";

    var model = openerp;
    var Qweb = openerp.web.qweb;

    widget.WidgetAccountReceivableWidget = widget.Base.extend({
        template: 'WidgetAccountReceivable',
        modelId: [],
        data: [],

        events: {
            'click .today': 'showToday',
            'click .thisWeek': 'showThisWeek',
            'click .expire': 'showExpire',
            'click .number': 'showModal', 
        },

        init: function (parent) {
            this._super(parent, {
                width: 3,
                height: 2
            });
        },

        start: function () {
            var self = this;
            self.fetchInitial();
        },

        fetchInitial: function(){
            var self = this;
            self.$el.find('#morosidad').block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#FAFAFA'
                }
            });
            self.$el.find('.widget-content.widget-loading').css('display','flex');

            self.fetchCurrentUser().then(function (CurrentUser) {
                return CurrentUser
            }).then(function(CurrentUser) {
                self.CurrentUser = CurrentUser;
                return self.fetchResUser(CurrentUser);
            }).then(function(ResUser) {
                self.ResUser = ResUser;
                return self.fetchAccountJournal();
            }).then(function(AccountJournal) {
                self.AccountJournal = AccountJournal;
                return self.fetchAccountMoveLine();
            }).then(function (AccountMoveLine) {
                self.AccountMoveLine = _.filter(AccountMoveLine, function(item) {
                    return item.amount_residual  > 0
                });
                return self.fetchAccountInvoice();
            }).then(function(AccountInvoice) {
                self.AccountInvoice = AccountInvoice;
                return self.fetchResCompany();
            }).then(function(ResCompany) {
                self.ResCompany = ResCompany;
                return self.fetchResCurrency();
            }).then(function(ResCurrency) {
                self.ResCurrency = ResCurrency;
                return self.fetchGetModelId();
            }).then(function(modelId) {
                self.modelId= modelId;
                return self.showExpire();
            });
        },

        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        fetchResUser: function(id) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','store_id'];
            var domain = [['id','=',id]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountJournal: function() {
            var self = this;
            var defer = $.Deferred();
            var store_ids = self.ResUser[0].store_id[0];
            var fields = ['id', 'name'];
            var domain = [['store_ids','in',store_ids]];
            var AccountJournal = new model.web.Model('account.journal');
            AccountJournal.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchAccountMoveLine: function() {
            var self = this;
            var defer = $.Deferred();
            var journal_ids = _.flatten(_.map(this.AccountJournal, function (item) {
                return item.id;
            }));
            var fields = ['id', 'partner_id', 'amount_residual', 'credit', 'debit', 'date_maturity', 'invoice','ref', 'invoice', 'amount_residual_currency', 'currency_id'];
            var domain = [['credit', '<=', 0],['date_maturity', '!=', false],['journal_id','in',journal_ids]];
            var AccountMoveLine = new model.web.Model('account.move.line');
            AccountMoveLine.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        fetchAccountInvoice: function() {
            var self = this;
            var defer = $.Deferred();
            var ref = _.map(self.AccountMoveLine, function(map) {
                return map.invoice[0];
            });
            var fields = ['id', 'currency_id', 'number', 'currency_id'];
            var domain = [['id', 'in', ref]];
            var AccountInvoice = new model.web.Model('account.invoice');
            AccountInvoice.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchResCompany: function() {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'currency_id'];
            var domain = [['id', '=', 1]];
            var ResCompany = new model.web.Model('res.company');
            ResCompany.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        fetchResCurrency : function(){
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name', 'symbol', 'rate_silent', 'base', 'decimal_separator', 'decimal_places', 'thousands_separator', 'symbol_position'];
            var domain = [['active', '=', true]];
            var ResCurrecy = new model.web.Model('res.currency');
            ResCurrecy.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });
            return defer;
        },

        // getModelId
        fetchGetModelId: function() {
            var self = this;
            var defer = $.Deferred();
            var irModelData = new model.web.Model('ir.model.data');
            var getObtjectReference = irModelData.get_func('get_object_reference');

            this.alive(getObtjectReference('account', 'invoice_form')).then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },

        getResCurrency: function (id) {
            var self = this;
            return _.filter(self.ResCurrency,function (item) {
                return item.id === id;
            })
        },

        getTodayAccountMoveLive:function() {
            var self = this;
            var date = moment().format('YYYY-MM-DD');
            return _.flatten(_.filter(self.AccountMoveLine,function (inv) {
                return moment(inv.date_maturity).format('YYYY-MM-DD') === date;
            }));
        },

        getThisWeekAccountMoveLive:function() {
            var self = this;
            var week = moment().week();
            return _.flatten(_.filter(self.AccountMoveLine,function (inv) {
                return moment(inv.date_maturity).week() === week & moment(inv.date_maturity).format('YYYY')=== moment().format('YYYY');
            }));
        },

        getExpireAccountMoveLive:function() {
            var self = this;
            return _.flatten(_.filter(self.AccountMoveLine,function (inv) {
                return moment(inv.date_maturity).format('YYYY-MM-DD') < moment().format('YYYY-MM-DD');
            }));
        },

        showToday: function () {
            var self = this;
            var amount = 0;
            // var data = [];
            var MoveLine = self.getTodayAccountMoveLive();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            if(MoveLine.length > 0){
                amount = _.reduce(_.map(MoveLine, function (map) {
                    return map.amount_residual;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            self.data = MoveLine;
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.$el.find('.widget-content').find('.number').text(accounting.formatMoney(amount, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator))
        },

        showThisWeek: function () {
            var self = this;
            var amount = 0;
            // var data = [];
            var MoveLine = self.getThisWeekAccountMoveLive();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            if(MoveLine.length > 0){
                amount = _.reduce(_.map(MoveLine, function (map) {
                    return map.amount_residual;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            self.data = MoveLine;
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.$el.find('.widget-content').find('.number').text(accounting.formatMoney(amount, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator))
        },

        showExpire: function () {
            var self = this;
            var amount = 0;
            var MoveLine = self.getExpireAccountMoveLive();
            var CurrencyBase = self.getResCurrency(self.ResCompany[0].currency_id[0]).shift();
            if(MoveLine.length > 0){
                amount = _.reduce(_.map(MoveLine, function (map) {
                    return map.amount_residual;
                }), function (memo, num) {
                    return memo + num;
                });
            }
            self.data = MoveLine;
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.$el.find('.widget-content').find('.number').text(accounting.formatMoney(amount, CurrencyBase.symbol, CurrencyBase.decimal_places, CurrencyBase.thousands_separator, CurrencyBase.decimal_separator))
        },

        showModal: function (e) {
            var self = this;
            if (self.data <=0) {
                model.web.notification.do_warn("Atención","Sin datos");
                return
            }

            var moneda = self.ResCurrency.shift();
            var newmoveLine = self.data;
            var titleData = [
                {
                    title: "Cuentas a Cobrar"
                }
            ];
            var headerModal = [
                {
                    title: "Cliente"
                },
                {
                    title: "Numero"
                },
                {
                    title: "Vencimiento"
                },
                {
                    title: "Monto "
                }
            ];

            var modal = Qweb.render('ExpiredAccountCountersModal', {
                data: newmoveLine,
                dataThead: headerModal,
                modalTitle: titleData
            });

            $('.openerp_webclient_container').after(modal);
            $('.expired-account-modal').modal()
            $('.expired-account-modal').on('hidden.bs.modal', function (e) {
                self.removeModal(e);
            })

            var contenido = $('.expired-account-modal').find('.table-tbody');
            contenido.click(function (e) {
                $(contenido).find('tr').removeClass('table-row-select');
                $(e.target).closest('tr').addClass('table-row-select');
                var chirdren_id =$(e.target).closest('tr').children()[0].textContent;
                self.renderForm(chirdren_id);
            });
        },

        removeModal: function (e) {
            $('.expired-account-modal').remove();
            $('.modal-backdrop').remove();
        },

        renderForm: function(id){
            var self = this;
            id = parseInt(id);

            this.do_action({
                type: "ir.actions.act_window",
                res_model: "account.invoice",
                views: [[self.modelId[1], 'form']],
                target: 'current',
                domain: [['id', '=', id]],
                context: {},
                res_id: id,
            }).then(function() {
                self.removeModal();
            });
        }
    });
}